# -*- coding: latin-1 -*-
# ###############################################################################
# Copyright © 2018 NetApp, Inc. All Rights Reserved.
#
# CONFIDENTIALITY NOTICE:  THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
# NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
# EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
###############################################################################

"""hcilogger"""


import logging
import logging.handlers
import sys
import re
from logging import Logger
from logtools import logtime
from logtools.config import Config

from logtools.config import Config

__author__ = "Joel Arthur"
__copyright__ = "Copyright © 2018, Inc. All Rights Reserved."
__version__ = "1.0"



#
# NOTE: Be careful as this file is pulled in by mnodecfg and must therefore work with python 2.7
#

PASSWORD_CAPTURE_REGEX = r"([a-zA-Z0-9_@\." + Config.PASSWORD_SPECIAL_CHARS + r"]+)"
JSON_CAPTURE_REGEX = r"([^'\\\"]*)"
URL_CAPTURE_REGEX = r"([^&]*)"


class SanitizeFilter(logging.Filter):
    """Filter usernames and passwords out of log messages."""

    def __init__(self):
        # Must use python2.7 syntax here - see note above
        super(logging.Filter, self).__init__()  # pylint: disable=bad-super-call
        self._json_patterns = []
        self._url_patterns = []
        self._disallowed_strings = []

        self._load_keyword('password')
        self._load_keyword('password_confirm')
        self._load_keyword('password-confirm')
        self._load_keyword('passwordConfirm')
        self._load_keyword('username')
        self._load_keyword('esxpass')
        self._load_keyword('sfuser')
        self._load_keyword('sfpass')
        self._load_keyword('targetSecret')
        self._load_keyword('initiatorSecret')
        self._load_keyword('ots_pass')
        self._load_keyword('Password')
        self._load_keyword('VCenterPw')
        self._load_keyword('MNodePw')
        self._load_keyword('mnode_password')

    def _load_keyword(self, keyword):
        self._json_patterns.append(self._json_kv_filter(keyword))
        self._url_patterns.append(self._url_kv_filter(keyword))

    @staticmethod
    def _json_kv_filter(key_name):
        return re.compile(r"(['\\\"]+_*" + key_name + r"['\\\"]+\s*:\s*['\\\"]+)" + JSON_CAPTURE_REGEX + r"(['\\\"]+[}\],\s]+)")

    @staticmethod
    def _url_kv_filter(key_name):
        return re.compile(r"(&?_*" + key_name + "=)" + PASSWORD_CAPTURE_REGEX + r"()")

    def set_disallowed_strings(self, disallowed_strings):
        """
        Set a list of strings that are not allowed to appear at all in log messages.  They will simple be
        removed any time they are encountered.  The most common use case is a catch-all for passwords.

        Args:
            disallowed_strings: Is a list of strings that are not allowed to be in log messages.
        """
        try:
            if not all(isinstance(s, str) for s in disallowed_strings):
                print('set_disallowed_strings: Ignoring {} - Not List of Strings'.format(disallowed_strings))
                return
        except Exception as e:  # pylint: disable=broad-except
            print('set_disallowed_strings: Ignoring {} - {}'.format(disallowed_strings, e))
            return

        self._disallowed_strings = disallowed_strings

    def filter(self, record):
        msg = str(record.msg)

        # Remove 'rootpw pass' used in kickstart
        msg = re.sub(r"^(.*rootpw\s+)" + PASSWORD_CAPTURE_REGEX + r"(.*)$", r"\1****\3", msg)

        for pattern in self._json_patterns:
            msg = pattern.sub(r"\1\3", msg)

        for pattern in self._url_patterns:
            msg = pattern.sub(r"\3", msg)
            msg = msg.replace('?&', '?')
            msg = msg.replace(r'&$', '')

        for _str in self._disallowed_strings:
            msg = msg.replace(_str, '')

        record.msg = msg

        return True


def __log_uncaught_exception(exctype, value, traceback):
    # This is the default printing to stderr
    sys.__excepthook__(exctype, value, traceback)
    # Also output to the sf-prime log
    logging.error('Uncaught Exception:', exc_info=(exctype, value, traceback))


SANITIZE_FILTER = SanitizeFilter()

# one-time setup of the root and sf-prime loggers
def setup_logger(log_name):

    # Setting the file and stream handlers on the root logger will allow them to be
    # propagated to all downstream loggers i.e. both the sf-prime logger and the loggers used by
    # other libraries such as flask.
    logging.basicConfig(format='%(asctime)s.%(msecs)03d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        level=Config.LOG_LEVEL)

    Config.LOG_NAME = log_name
    root_logger = logging.getLogger()
    root_logger.addFilter(SANITIZE_FILTER)
    root_logger.setLevel(level=Config.LOG_LEVEL)

    # The logtime utc filter must be set on the handlers instead of the logger as filters are not propagated
    # and we would see a log format error trying to access the utc key on downstream loggers.
    utc_filter = logtime.ContextFilter()

    authtools_logger = logging.getLogger('authtools')
    authtools_logger.setLevel(level=Config.LOG_LEVEL)

    # Setup the actual HCI logger
    hci_logger = logging.getLogger(Config.LOG_NAME)
    hci_logger.setLevel(level=Config.LOG_LEVEL)
    hci_logger.addFilter(SANITIZE_FILTER)
    hci_logger.addFilter(utc_filter)
    hci_logger_level = logging.getLevelName(hci_logger.getEffectiveLevel())
    logging.info(f"Log level for {Config.LOG_NAME} has been set to {hci_logger_level}")

    # Events which have been generated by descendant loggers will not be filtered by a logger?s filter setting,
    # unless the filter has also been applied to those descendant loggers.
    # see: https://docs.python.org/3.5/library/logging.html#filter-objects
    # Because the SDK sets up it's own logger we need to apply the sanitize filter to it as well
    # ------- This is removed because all SolidFire SDK references are pulled out for now. --------
    # try:
    #     import solidfire
    #     sdk_logger = solidfire.common.LOG
    #     sdk_logger.addFilter(SANITIZE_FILTER)
    #     sdk_stream_handler = solidfire.common.CH
    #     sdk_stream_handler.addFilter(SANITIZE_FILTER)
    # except Exception:  # pylint: disable=broad-except
    #     sfprime_logger.exception('Failed to import solidfire sdk - this is normal only for the mnodecfg.py')

    # Setup the uncaught exception logger
    sys.excepthook = __log_uncaught_exception

    return hci_logger



