# -*- coding: latin-1 -*-
###############################################################################
# Copyright © 2018 NetApp, Inc. All Rights Reserved.
#
# CONFIDENTIALITY NOTICE:  THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
# NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
# EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
###############################################################################

"""
NAME
  logtime.py

DESCRIPTION
  logging filter
"""

import os
import sys
import time
import ctypes
import logging

__author__ = "John Patterson"
__copyright__ = "Copyright © 2018 NetApp, Inc. All Rights Reserved."
__version__ = "1.0"

CLOCK_REALTIME = 0  # see <linux/time.h>


class TimeSpec(ctypes.Structure):
    '''Class to get precise timer'''
    _fields_ = [
        ('tv_sec', ctypes.c_long),
        ('tv_nsec', ctypes.c_long)
    ]


if os.name == 'posix':
    if sys.platform == 'darwin':
        LIBRT = ctypes.CDLL('/usr/lib/libc.dylib', use_errno=True)
    else:
        LIBRT = ctypes.CDLL('librt.so.1', use_errno=True)
    CLOCK_GETTIME = LIBRT.clock_gettime
    CLOCK_GETTIME.argtypes = [ctypes.c_int, ctypes.POINTER(TimeSpec)]


def clock_realtime():
    '''Pull realtime from linux/time - this is used to present the same time format in Python logs as we do in Element'''
    if os.name == 'posix':
        ttime = TimeSpec()
        if CLOCK_GETTIME(CLOCK_REALTIME, ctypes.pointer(ttime)) != 0:
            errno_ = ctypes.get_errno()
            raise OSError(errno_, os.strerror(errno_))
        return ttime.tv_sec + ttime.tv_nsec * 1e-9
    return 0


class ContextFilter(logging.Filter):
    """
    This is a filter class which injects contextual information into the _LOG.
    """

    def filter(self, record):
        now = clock_realtime()
        micros = (now - int(now)) * 1000000.0
        record.utc = time.strftime("%Y-%m-%dT%H:%M:%S", time.gmtime(now)) + ".{:06.0f}Z".format(micros)
        return True
