import logging
import os
from statustools.config import Config
from statustools import constants
from tenacity import retry, wait_fixed, stop_after_attempt, retry_if_exception_type
from redis.exceptions import ConnectionError


_LOG = logging.getLogger('statustools')


@retry(reraise=True, stop=stop_after_attempt(3), wait=wait_fixed(1), retry=retry_if_exception_type(ConnectionError))
def cache_update(key: str, value: str) -> None:
    """update a value in the cache

    Arguments:
        key {str} -- the key
        value {str} -- the calue

    Raises:
        TypeError: If an invalid type of CACHE_STATUS is used

    Returns:
        None
    """
    try:
        lock = Config.REDIS_CLIENT.lock('redis_db_lock')
        _LOG.info(f"Setting {key} to: {value}")
        with lock:
            if key == constants.CACHE_STATUS:
                if isinstance(value, constants.ServiceUpdateStatusEnum):
                    return Config.REDIS_CLIENT.set(key, value.value)
                raise TypeError(f'{key} is not of type ServiceUpdateStatusEnum')
            else:
                value = str(value)
                Config.REDIS_CLIENT.set(key, value.strip())
    except TypeError as e:
        raise e
    except ConnectionError as e:
        _LOG.warning("Can not communicate with REDIS.")
        raise e

@retry(reraise=True, stop=stop_after_attempt(3), wait=wait_fixed(1), retry=retry_if_exception_type(ConnectionError))
def cache_get(key: str, default: str = None) -> str:
    """Get a value from the cache

    Arguments:
        key {str} -- the Key

    Returns:
        str -- The value
    """
    try:
        lock = Config.REDIS_CLIENT.lock('redis_db_lock')
        _LOG.info(f"Getting {key}")
        with lock:
            value = Config.REDIS_CLIENT.get(key)
            if isinstance(value, bytes):
                value = value.decode()
        value = default if not value else value
        _LOG.info(f"{key} = {value}")
        return value
    except ConnectionError as e:
        _LOG.warning("Can not communicate with REDIS.")
        raise e

@retry(reraise=True, stop=stop_after_attempt(3), wait=wait_fixed(1), retry=retry_if_exception_type(ConnectionError))
def cache_delete(key: str) -> None:
    """Delete a value from the cache

    Arguments:
        key {str} -- The key to delete

    Returns:
        None
    """
    try:
        lock = Config.REDIS_CLIENT.lock('redis_db_lock')
        _LOG.info(f"Deleting key: {key}")
        with lock:
            if Config.REDIS_CLIENT.exists(key):
                Config.REDIS_CLIENT.delete(key)
    except ConnectionError as e:
        _LOG.warning("Can not communicate with REDIS.")
        raise e

def cache_clear() -> None:
    """Clear out all cache entries

    Returns:
        None
    """
    _LOG.info(f"Clearing cache")
    cache_delete(constants.CACHE_STATUS)
    cache_delete(constants.CACHE_DETAILS)
    cache_delete(constants.CACHE_UPDATING_TO_VERSION)
    cache_delete(constants.CACHE_UPDATE_PACKAGE_SIZE)
    cache_delete(constants.CACHE_UPDATE_START_TIME)
    cache_delete(constants.CACHE_UPDATE_DOWNLOADED_SIZE)
    cache_delete(constants.CACHE_PERCENTAGE_COMPLETE)
    cache_delete(constants.CACHE_MNODE_BUNDLE_VERSION)

def get_status() -> dict:
    """Get the current status

    Returns:
        dict -- The status
    """
    status = cache_get(constants.CACHE_STATUS)
    details = cache_get(constants.CACHE_DETAILS)
    update_package_size = cache_get(constants.CACHE_UPDATE_PACKAGE_SIZE)
    update_start_time = cache_get(constants.CACHE_UPDATE_START_TIME)
    updating_to_version = cache_get(constants.CACHE_UPDATING_TO_VERSION)
    update_downloaded_size = cache_get(constants.CACHE_UPDATE_DOWNLOADED_SIZE)
    update_percentage_complete = cache_get(constants.CACHE_PERCENTAGE_COMPLETE)
    current_version = cache_get(constants.CACHE_MNODE_BUNDLE_VERSION)

    if not status:
        status = constants.ServiceUpdateStatusEnum.NOT_STARTED.value
        update_percentage_complete = 0

    response = {'status': status, 'details': details, 'current_version': current_version}

    if updating_to_version:
        response['updating_to_version'] = updating_to_version
        if update_downloaded_size:
            try:
                response['downloaded'] = int(update_downloaded_size)
            except ValueError:
                response['downloaded'] = 0
    if update_package_size:
        try:
            response['total_size'] = int(update_package_size)
        except ValueError:
            response['total_size'] = 0
    if update_start_time:
        response['update_start_time'] = update_start_time

    if update_percentage_complete:
        try:
            response['update_percentage_complete'] = int(update_percentage_complete)
        except ValueError:
            response['update_percentage_complete'] = 0
    else:
        response['update_percentage_complete'] = 0

    return response